"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const performance_now_1 = tslib_1.__importDefault(require("performance-now"));
const fast_stats_1 = require("fast-stats");
const Event_1 = require("./Event");
const { Phase, Subject, Type } = Event_1.Event;
const DEFAULT_CONFIG = {
    verifyPasses: 2,
    warmups: 10,
    minSamples: 25,
    maxDurationMs: 15 /* seconds */ * 1e3,
    targetRelativeMarginOfError: 5.0,
};
function runSuite(reporter, benchmarkClasses, clientClasses, rawExample, config = DEFAULT_CONFIG) {
    const context = {
        reporter,
        config,
        rawExample,
        eventCommon: {
            clients: clientClasses.map(c => c.metadata),
            benchmarks: benchmarkClasses.map(b => b.metadata),
            example: rawExample,
            canceled: false,
        },
    };
    const innerPromise = _runSuite(context, benchmarkClasses, clientClasses);
    const promise = innerPromise.then(() => {
        return context.canceled;
    });
    promise.cancel = function cancel() {
        context.canceled = true;
        context.eventCommon.canceled = true;
    };
    return promise;
}
exports.runSuite = runSuite;
async function _runSuite(context, benchmarkClasses, clientClasses) {
    const { reporter, rawExample } = context;
    reporter(Object.assign({}, context.eventCommon, { subject: Subject.SUITE, type: Type.START }));
    for (const BenchmarkClass of benchmarkClasses) {
        if (context.canceled)
            break;
        reporter(Object.assign({}, context.eventCommon, { subject: Subject.BENCHMARK, type: Type.START, benchmark: BenchmarkClass.metadata }));
        for (const ClientClass of clientClasses) {
            if (context.canceled)
                break;
            await runClientBenchmark(context, BenchmarkClass, ClientClass);
        }
        reporter(Object.assign({}, context.eventCommon, { subject: Subject.BENCHMARK, type: Type.END, benchmark: BenchmarkClass.metadata }));
    }
    reporter(Object.assign({}, context.eventCommon, { subject: Subject.SUITE, type: Type.END }));
}
exports._runSuite = _runSuite;
async function runClientBenchmark(context, BenchmarkClass, ClientClass) {
    const { reporter, rawExample, config } = context;
    const eventCommon = Object.assign({}, context.eventCommon, { benchmark: BenchmarkClass.metadata, client: ClientClass.metadata });
    reporter(Object.assign({}, eventCommon, { subject: Subject.CLIENT_BENCHMARK, type: Type.START }));
    const client = new ClientClass();
    const { title, schema, partials } = rawExample;
    // Preprocess & Verify
    reporter(Object.assign({}, eventCommon, { subject: Subject.CLIENT_BENCHMARK_PHASE, phase: Phase.VERIFY, type: Type.START }));
    const verifyStart = performance_now_1.default();
    let example;
    try {
        const rootExample = client.transformRawExample(rawExample, schema);
        example = Object.assign({}, rootExample, { title, partials: partials.map(p => client.transformRawExample(p, schema)) });
    }
    catch (error) {
        context.failure = { error, phase: Phase.VERIFY };
    }
    for (let i = 0; i < config.verifyPasses; i++) {
        if (context.canceled || context.failure)
            break;
        await runSingleBenchmarkPass(context, BenchmarkClass, client, example, Phase.VERIFY, {
            verify: true,
        });
    }
    reporter(Object.assign({}, eventCommon, { subject: Subject.CLIENT_BENCHMARK_PHASE, phase: Phase.VERIFY, type: Type.END, duration: performance_now_1.default() - verifyStart, failure: context.failure }));
    // Brief pause to give the UI (and maybe GC) a chance to catch up.
    await new Promise(resolve => setTimeout(resolve, 50));
    // Warmup
    if (!context.canceled && !context.failure) {
        reporter(Object.assign({}, eventCommon, { subject: Subject.CLIENT_BENCHMARK_PHASE, phase: Phase.WARMUP, type: Type.START }));
        const warmupStart = performance_now_1.default();
        for (let i = 0; i < config.warmups; i++) {
            if (context.canceled || context.failure)
                break;
            await runSingleBenchmarkPass(context, BenchmarkClass, client, example, Phase.WARMUP);
        }
        reporter(Object.assign({}, eventCommon, { subject: Subject.CLIENT_BENCHMARK_PHASE, phase: Phase.WARMUP, type: Type.END, duration: performance_now_1.default() - warmupStart, failure: context.failure }));
    }
    // Brief pause to give the UI (and maybe GC) a chance to catch up.
    await new Promise(resolve => setTimeout(resolve, 50));
    // Iterations
    const stats = new fast_stats_1.Stats();
    const runStart = performance_now_1.default();
    if (!context.canceled && !context.failure) {
        while (true) {
            if (context.canceled || context.failure)
                break;
            reporter(Object.assign({}, eventCommon, { subject: Subject.CLIENT_BENCHMARK_PHASE, phase: Phase.ITERATION, type: Type.START }));
            const duration = await runSingleBenchmarkPass(context, BenchmarkClass, client, example, Phase.ITERATION);
            if (typeof duration !== 'undefined') {
                stats.push(duration);
            }
            reporter(Object.assign({}, eventCommon, { subject: Subject.CLIENT_BENCHMARK_PHASE, phase: Phase.ITERATION, type: Type.END, duration, stats: statsSummary(stats), failure: context.failure }));
            if (isFinalIteration(config, runStart, stats))
                break;
        }
    }
    // Brief pause to give the UI (and maybe GC) a chance to catch up.
    await new Promise(resolve => setTimeout(resolve, 50));
    // Remove outliers.
    const finalStats = statsSummary(stats.iqr());
    reporter(Object.assign({}, eventCommon, { subject: Subject.CLIENT_BENCHMARK, type: Type.END, stats: finalStats, failure: context.failure }));
    // And we're done with the failure
    context.failure = undefined;
    return finalStats;
}
function isFinalIteration(config, runStart, stats) {
    if (stats.length < config.minSamples)
        return false;
    if (performance_now_1.default() - runStart > config.maxDurationMs)
        return true;
    if (percentRelativeMarginOfError(stats) <= config.targetRelativeMarginOfError)
        return true;
    return false;
}
async function runSingleBenchmarkPass(context, BenchmarkClass, client, example, phase, { verify = false } = {}) {
    try {
        const benchmark = new BenchmarkClass(client, example);
        await benchmark.setup();
        const start = performance_now_1.default();
        await benchmark.run();
        const duration = performance_now_1.default() - start;
        if (verify) {
            await benchmark.verify();
        }
        await benchmark.teardown();
        // Yield the run loop.
        await new Promise(resolve => setTimeout(resolve, 0));
        return duration;
    }
    catch (error) {
        context.failure = { error, phase };
        return undefined;
    }
}
function percentRelativeMarginOfError(stats) {
    return (stats.moe() / stats.amean()) * 100;
}
function statsSummary(stats) {
    const [min, max] = stats.range();
    return {
        iterations: stats.length,
        min,
        mean: stats.amean(),
        max,
        marginOfError: stats.moe(),
        percentRelativeMarginOfError: percentRelativeMarginOfError(stats),
    };
}
