"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const apollo_utilities_1 = require("apollo-utilities");
const fast_random_1 = tslib_1.__importDefault(require("fast-random"));
const graphql_tag_1 = tslib_1.__importDefault(require("graphql-tag"));
const printer_1 = require("graphql/language/printer"); // eslint-disable-line import/no-internal-modules
// Use a consistent seed for consistent partials for a given query.
const SEED = 12345678;
const NUM_EXAMPLES = 8;
/**
 * Create 8 partial instances of the operation, by selecting a linearly
 * decreasing % of fields.
 *
 * E.g. the first example will have 100% of the fields, the second 87.5%, and so
 * on.
 */
function generatePartialExamples(example) {
    const document = graphql_tag_1.default(example.operation);
    const operation = apollo_utilities_1.getOperationDefinitionOrDie(document);
    const leaves = findLeafPaths(document);
    const random = fast_random_1.default(SEED);
    const partials = [];
    for (let i = NUM_EXAMPLES; i > 0; i--) {
        const selectPercent = i / NUM_EXAMPLES;
        const partialLeaves = selectLeaves(random, leaves, selectPercent);
        const partialSelectionSet = selectionSetFromLeaves(partialLeaves);
        const partialOperation = {
            kind: 'OperationDefinition',
            name: { kind: 'Name', value: `partial${i}` },
            selectionSet: partialSelectionSet,
            directives: operation.directives,
            operation: operation.operation,
            // TODO: Remove unused variables.
            variableDefinitions: operation.variableDefinitions,
        };
        partials.push({
            operation: printer_1.print(partialOperation),
            variables: example.variables,
            response: reduceResponse(partialSelectionSet, example.response),
        });
    }
    return partials;
}
exports.generatePartialExamples = generatePartialExamples;
function findLeafPaths(document) {
    const leaves = [];
    const operation = apollo_utilities_1.getOperationDefinitionOrDie(document);
    const fragmentMap = apollo_utilities_1.createFragmentMap(apollo_utilities_1.getFragmentDefinitions(document));
    walkSelectionSetForLeaves(leaves, fragmentMap, operation.selectionSet, []);
    return leaves;
}
function walkSelectionSetForLeaves(leaves, fragmentMap, selectionSet, parents) {
    for (const selection of selectionSet.selections) {
        if (selection.kind === 'Field') {
            const path = [...parents, selection];
            if (selection.selectionSet) {
                // we've found a branch node.
                walkSelectionSetForLeaves(leaves, fragmentMap, selection.selectionSet, path);
            }
            else {
                // We've found a leaf.
                leaves.push(path);
            }
        }
        else if (selection.kind === 'FragmentSpread') {
            const fragment = fragmentMap[selection.name.value];
            if (!fragment) {
                throw new Error(`Expected fragment ${selection.name.value} to be defined`);
            }
            walkSelectionSetForLeaves(leaves, fragmentMap, fragment.selectionSet, parents);
        }
    }
}
function selectLeaves(random, leaves, selectPercent) {
    const toKeep = [];
    const toShuffle = [];
    for (const leaf of leaves) {
        const fieldName = leaf[leaf.length - 1].name.value;
        // To appease Apollo caches, preserve entity identities.
        if (fieldName === 'id' || fieldName === '__typename') {
            toKeep.push(leaf);
        }
        else {
            toShuffle.push(leaf);
        }
    }
    const shuffled = [];
    while (toShuffle.length) {
        const index = Math.floor(random.nextFloat() * toShuffle.length);
        const leaf = toShuffle.splice(index, 1)[0];
        shuffled.push(leaf);
    }
    const toSelect = Math.ceil(selectPercent * leaves.length);
    return shuffled.slice(0, toSelect).concat(toKeep);
}
function selectionSetFromLeaves(leaves) {
    const selections = [];
    for (const leaf of leaves) {
        mergeLeaf(selections, leaf);
    }
    return { kind: 'SelectionSet', selections };
}
function mergeLeaf(selections, leaf) {
    const [top, ...remainder] = leaf;
    // last field? we're by definition new to the selections
    if (leaf.length == 1) {
        selections.push(top);
        return;
    }
    let ancestor = selections.find(s => s.name.value === top.name.value);
    if (!ancestor) {
        ancestor = {
            kind: 'Field',
            alias: top.alias,
            name: top.name,
            arguments: top.arguments,
            directives: top.directives,
            selectionSet: {
                kind: 'SelectionSet',
                selections: [],
            },
        };
        selections.push(ancestor);
    }
    mergeLeaf(ancestor.selectionSet.selections, remainder);
}
function reduceResponse(selectionSet, response) {
    if (!response)
        return response;
    if (Array.isArray(response)) {
        return response.map(v => reduceResponse(selectionSet, v));
    }
    const reducedResponse = {};
    for (const selection of selectionSet.selections) {
        const key = selection.alias ? selection.alias.value : selection.name.value;
        if (selection.selectionSet) {
            reducedResponse[key] = reduceResponse(selection.selectionSet, response[key]);
        }
        else {
            reducedResponse[key] = response[key];
        }
    }
    return reducedResponse;
}
