"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const dapper = tslib_1.__importStar(require("@convoy/dapper"));
const benchmarks_1 = tslib_1.__importDefault(require("../../../benchmarks"));
const clients_1 = tslib_1.__importDefault(require("../../../clients"));
const examples_1 = tslib_1.__importDefault(require("../../../examples"));
const reporting_1 = require("../../reporting");
const execution_1 = require("../../execution");
const SuiteSummary_1 = require("./SuiteSummary");
const ExampleEditor_1 = require("./ExampleEditor");
// Update at a lowish framerate to give priority to benchmarks.
const UPDATE_DELAY_MS = 50;
var RunState;
(function (RunState) {
    RunState[RunState["PENDING"] = 0] = "PENDING";
    RunState[RunState["RUNNING"] = 1] = "RUNNING";
    RunState[RunState["CANCELED"] = 2] = "CANCELED";
    RunState[RunState["COMPLETE"] = 3] = "COMPLETE";
})(RunState || (RunState = {}));
const MODES = {
    pending: ({ state }) => !state || state.runState === RunState.PENDING,
    running: ({ state }) => state && state.runState === RunState.RUNNING,
    canceled: ({ state }) => state && state.runState === RunState.CANCELED,
    complete: ({ state }) => state && state.runState === RunState.COMPLETE,
    idle: ({ state }) => !state || state.runState !== RunState.RUNNING,
};
const STYLES = dapper.compile({
    global: {
        fontFamily: `"Proxima Nova", ProximaNova, "Helvetica Neue", Helvetica, sans-serif`,
        fontSize: 16,
        lineHeight: '22px',
    },
    root: {
        border: `3px solid #15455a`,
        backgroundColor: '#0b2127',
        color: '#ffffff',
    },
    header: {
        display: 'flex',
        background: 'linear-gradient(to bottom, #15455a, #0b2127)',
        color: '#ffffff',
        padding: 10,
        fontSize: 20,
        lineHeight: '28px',
        'button, select': {
            fontSize: 'inherit',
            lineHeight: 'inherit',
            backgroundColor: 'transparent',
            color: '#ffffff',
        },
    },
    status: {
        $running: {
            color: '#5b666b',
        },
        $complete: {
            color: '#4aa54f',
        },
        $canceled: {
            color: '#f8cc1c',
        },
    },
    about: {
        flex: 1,
    },
    controls: {
        display: 'flex',
        flexDirection: 'column',
        alignSelf: 'stretch',
        alignItems: 'flex-end',
    },
    buttons: {
        button: {
            marginLeft: 14,
        },
    },
    exampleSelectorContainer: {
        flex: 1,
        marginBottom: 14,
    },
    title: {
        fontSize: 24,
        lineHeight: '34px',
    },
    toggle: {
        $idle: {
            backgroundColor: '#f7f9fa',
            color: '#125571',
        },
    },
    summary: {
        $canceled: {
            opacity: 0.5,
        },
    },
});
class Root extends React.PureComponent {
    constructor() {
        super(...arguments);
        this.styles = dapper.reactTo(this, STYLES, MODES);
        this.state = {
            runState: RunState.PENDING,
            editing: false,
            examples: examples_1.default,
            exampleIndex: 0,
        };
        this._onToggleSuite = () => {
            const { runState } = this.state;
            if (runState === RunState.RUNNING) {
                this.stop();
            }
            else {
                this.start();
            }
        };
        this._onEditCurrentExample = () => {
            this.setState({ editing: true });
        };
        this._onCurrentExampleEdited = (newExample) => {
            const { examples, exampleIndex } = this.state;
            const newExamples = [...examples];
            newExamples[exampleIndex] = newExample;
            this.setState({ examples: newExamples, editing: false });
        };
        this._onEditorCanceled = () => {
            this.setState({ editing: false });
        };
        this._onChangeExample = event => {
            const { examples } = this.state;
            this.stop(); // Always.
            const exampleIndex = parseInt(event.target.value);
            // Are we adding a new one?
            if (exampleIndex >= examples.length) {
                this.setState({ exampleIndex, editing: true });
            }
            else {
                this.setState({ exampleIndex });
            }
        };
    }
    componentDidMount() {
        if (module.hot) {
            module.hot.dispose(() => {
                this._suitePromise.cancel();
            });
        }
    }
    render() {
        if (this._updateTimer) {
            clearTimeout(this._updateTimer);
            this._updateTimer = null;
        }
        return (React.createElement("div", { className: this.styles.global },
            React.createElement("div", { className: this.styles.root },
                this._renderHeader(),
                !!this._summary && React.createElement(SuiteSummary_1.SuiteSummary, Object.assign({}, this._summary, { className: this.styles.summary }))),
            this._maybeRenderEditor()));
    }
    _renderHeader() {
        const { runState } = this.state;
        return (React.createElement("header", { className: this.styles.header },
            React.createElement("div", { className: this.styles.about },
                React.createElement("div", { className: this.styles.title }, "GraphQL Client Benchmark Suite"),
                React.createElement("div", { className: this.styles.status }, this._statusLabel())),
            React.createElement("div", { className: this.styles.controls },
                React.createElement("div", { className: this.styles.exampleSelectorContainer },
                    "Example Query: ",
                    this._renderExampleSelector()),
                React.createElement("div", { className: this.styles.buttons },
                    React.createElement("button", { onClick: this._onEditCurrentExample }, "Edit"),
                    React.createElement("button", { onClick: this._onToggleSuite, className: this.styles.toggle }, runState === RunState.RUNNING ? 'Stop' : 'Start')))));
    }
    _statusLabel() {
        const { runState } = this.state;
        if (runState === RunState.RUNNING) {
            return 'running…';
        }
        else if (runState === RunState.CANCELED) {
            return 'canceled';
        }
        else if (runState === RunState.COMPLETE) {
            return 'benchmark suite complete';
        }
        else {
            return null;
        }
    }
    _renderExampleSelector() {
        const { examples, exampleIndex } = this.state;
        return (React.createElement("select", { className: this.styles.exampleSelector, value: exampleIndex, onChange: this._onChangeExample },
            examples.map((example, index) => (React.createElement("option", { value: index, key: index }, example.title))),
            React.createElement("option", { value: examples.length }, "Add Query\u2026")));
    }
    _maybeRenderEditor() {
        const { editing, examples, exampleIndex } = this.state;
        if (!editing)
            return null;
        const example = examples[exampleIndex] || {
            title: `New Example`,
            operation: '',
            schema: '',
            response: {},
            partials: [],
        };
        return (React.createElement(ExampleEditor_1.ExampleEditor, { example: example, onChange: this._onCurrentExampleEdited, onCancel: this._onEditorCanceled }));
    }
    start() {
        const { examples, exampleIndex } = this.state;
        if (this._suitePromise) {
            // Cancel, wait for it, and have the user press again.
            this.stop();
            return;
        }
        this._collector = new reporting_1.SuiteSummaryCollector(summary => {
            this._summary = summary;
            if (this._updateTimer)
                return;
            this._updateTimer = setTimeout(() => this.forceUpdate(), UPDATE_DELAY_MS);
        }, ({ benchmark, client, phase, error }) => {
            console.error(`Error during ${phase} of`, { benchmark, client }, error);
        });
        const example = examples[exampleIndex];
        this._suitePromise = execution_1.runSuite(this._collector.consumeEvent, benchmarks_1.default, clients_1.default, example);
        this._suitePromise.then(canceled => {
            this.setState({
                runState: canceled ? RunState.CANCELED : RunState.COMPLETE,
            });
            this._suitePromise = null;
        });
        this.setState({
            runState: RunState.RUNNING,
        });
    }
    stop() {
        if (!this._suitePromise)
            return;
        this._suitePromise.cancel();
    }
}
exports.Root = Root;
