"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Event_1 = require("./Event");
const { Subject, Type, Phase } = Event_1.Event;
/**
 * A helper that collects suite run summaries for tabular display.
 */
class SuiteSummaryCollector {
    constructor(onUpdate, onError) {
        this.onUpdate = onUpdate;
        this.onError = onError;
        /**
         * Must be called with every event emitted by the suite runner (a reporter).
         */
        this.consumeEvent = (event) => {
            if (event.subject === Subject.SUITE && event.type === Type.START) {
                this._summary = {
                    title: event.example.title,
                    benchmarks: event.benchmarks,
                    clients: event.clients,
                    running: true,
                    canceled: false,
                    results: {},
                };
                this.onUpdate(this._summary);
            }
            else if (event.subject === Subject.CLIENT_BENCHMARK_PHASE && event.type === Type.START) {
                this._updateBenchmarkResult(event, { phase: event.phase });
            }
            else if (event.subject === Subject.CLIENT_BENCHMARK_PHASE &&
                event.phase === Phase.ITERATION &&
                event.type === Type.END) {
                this._updateBenchmarkResult(event, { stats: event.stats });
            }
            else if (event.subject === Subject.CLIENT_BENCHMARK_PHASE && event.type === Type.END && event.failure) {
                if (this.onError) {
                    this.onError({
                        benchmark: event.benchmark.title,
                        client: event.client.name,
                        phase: event.phase,
                        error: event.failure.error,
                    });
                }
            }
            else if (event.subject === Subject.CLIENT_BENCHMARK && event.type === Type.END) {
                this._updateBenchmarkResult(event, { complete: true, failure: event.failure });
            }
            else if (event.subject === Subject.BENCHMARK && event.type === Type.END) {
                const { benchmark, clients } = event;
                const benchmarkResults = this._summary.results[benchmark.title];
                let minMean = Infinity;
                let maxMean = 0;
                for (const client of clients) {
                    const clientResults = benchmarkResults[client.name];
                    if (!clientResults || !clientResults.stats)
                        return; // We interrupted the benchmark
                    const { mean } = clientResults.stats;
                    minMean = Math.min(mean, minMean);
                    maxMean = Math.max(mean, maxMean);
                }
                for (const client of clients) {
                    const { mean } = benchmarkResults[client.name].stats;
                    this._updateBenchmarkResult({ benchmark, client }, {
                        percentOfRange: (mean - minMean) / (maxMean - minMean),
                    });
                }
            }
            else if (event.subject === Subject.SUITE && event.type === Type.END) {
                this._summary.running = false;
                this._summary.canceled = event.canceled;
                this.onUpdate(this._summary);
            }
        };
    }
    _updateBenchmarkResult({ benchmark, client }, update) {
        const { results } = this._summary;
        this._summary = Object.assign({}, this._summary, { results: Object.assign({}, results, { [benchmark.title]: Object.assign({}, results[benchmark.title], { [client.name]: Object.assign({}, (results[benchmark.title] && results[benchmark.title][client.name]), update) }) }) });
        this.onUpdate(this._summary);
    }
}
exports.SuiteSummaryCollector = SuiteSummaryCollector;
